@@ -1,24 +1,16 @@
-# Copyright (c) 2013 Amazon.com, Inc. or its affiliates.  All Rights Reserved
+# Copyright 2012-2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the
-# "Software"), to deal in the Software without restriction, including
-# without limitation the rights to use, copy, modify, merge, publish, dis-
-# tribute, sublicense, and/or sell copies of the Software, and to permit
-# persons to whom the Software is furnished to do so, subject to the fol-
-# lowing conditions:
+# Licensed under the Apache License, Version 2.0 (the "License"). You
+# may not use this file except in compliance with the License. A copy of
+# the License is located at
 #
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
-# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
-# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
-# IN THE SOFTWARE.
+# http://aws.amazon.com/apache2.0/
 #
+# or in the "license" file accompanying this file. This file is
+# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
+# ANY KIND, either express or implied. See the License for the specific
+# language governing permissions and limitations under the License.
+
 from itertools import tee
 try:
     from itertools import zip_longest
@@ -105,18 +97,22 @@ class Paginator(object):
 
 class PageIterator(object):
     def __init__(self, operation, input_token, output_token, more_results,
-                 result_key, max_items, starting_token, endpoint, op_kwargs):
+                 result_keys, max_items, starting_token, endpoint, op_kwargs):
         self._operation = operation
         self._input_token = input_token
         self._output_token = output_token
         self._more_results = more_results
-        self._result_key = result_key
+        self._result_keys = result_keys
         self._max_items = max_items
         self._starting_token = starting_token
         self._endpoint = endpoint
         self._op_kwargs = op_kwargs
         self._resume_token = None
 
+    @property
+    def result_keys(self):
+        return self._result_keys
+
     @property
     def resume_token(self):
         """Token to specify to resume pagination."""
@@ -135,7 +131,7 @@ class PageIterator(object):
         # The number of items from result_key we've seen so far.
         total_items = 0
         first_request = True
-        primary_result_key = self._result_key[0]
+        primary_result_key = self.result_keys[0]
         starting_truncation = 0
         self._inject_starting_token(current_kwargs)
         while True:
@@ -205,7 +201,7 @@ class PageIterator(object):
         # We also need to truncate any secondary result keys
         # because they were not truncated in the previous last
         # response.
-        for token in self._result_key:
+        for token in self.result_keys:
             if token == primary_result_key:
                 continue
             parsed[token] = []
@@ -241,9 +237,9 @@ class PageIterator(object):
         return next_tokens
 
     def result_key_iters(self):
-        teed_results = tee(self, len(self._result_key))
+        teed_results = tee(self, len(self.result_keys))
         return [ResultKeyIterator(i, result_key) for i, result_key
-                in zip(teed_results, self._result_key)]
+                in zip(teed_results, self.result_keys)]
 
     def build_full_result(self):
         iterators = self.result_key_iters()
